package com.totopia.server.init;

import com.totopia.server.modules.dashboard.repository.DashboardRepository;
import com.totopia.server.modules.user.entity.GradeEntity;
import com.totopia.server.modules.user.entity.RoleEntity;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.repository.GradeRepository;
import com.totopia.server.modules.user.repository.RoleRepository;
import com.totopia.server.modules.user.repository.UserRepository;
import com.totopia.server.modules.user.type.GradeName;
import com.totopia.server.modules.user.type.RoleName;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
@ConditionalOnProperty(name = "app.db-init", havingValue = "true")
@Slf4j
public class DbInitializer implements CommandLineRunner {

  @Autowired
  private RoleRepository roleRepository;

  @Autowired
  private UserRepository userRepository;

  @Autowired
  PasswordEncoder passwordEncoder;

  @Autowired
  DashboardRepository dashboardRepository;

  @Autowired
  GradeRepository gradeRepository;

  @Override
  public void run(String... strings) throws Exception {
    if (0 == roleRepository.count()) {
      RoleEntity role;
      role = RoleEntity.builder().name(RoleName.ROLE_USER).build();
      roleRepository.save(role);
      role = RoleEntity.builder().name(RoleName.ROLE_TEAM).build();
      roleRepository.save(role);
      role = RoleEntity.builder().name(RoleName.ROLE_AGENCY).build();
      roleRepository.save(role);
      role = RoleEntity.builder().name(RoleName.ROLE_ADMIN).build();
      roleRepository.save(role);
      role = RoleEntity.builder().name(RoleName.ROLE_SUPER_ADMIN).build();
      roleRepository.save(role);
    }

    if (0 == gradeRepository.count()) {
      GradeEntity grade;
      grade = GradeEntity.builder().name(GradeName.BAD_USER).build();
      gradeRepository.save(grade);
      grade = GradeEntity.builder().name(GradeName.NORMAL_USER).build();
      gradeRepository.save(grade);
      grade = GradeEntity.builder().name(GradeName.REGULAR_USER).build();
      gradeRepository.save(grade);
      grade = GradeEntity.builder().name(GradeName.VIP_USER).build();
      gradeRepository.save(grade);
      grade = GradeEntity.builder().name(GradeName.PLATINUM_USER).build();
      gradeRepository.save(grade);
    }

    if (0 == userRepository.count()) {
      UserEntity user;
      user = UserEntity.builder().username("admin").password(passwordEncoder.encode("admin")).nickname("admin")
          .email("admin@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 1)).build())
              .collect(Collectors.toCollection(HashSet::new)))
          .build();

      userRepository.save(user);
      user = UserEntity.builder().username("test1").password(passwordEncoder.encode("qwer5795")).nickname("test1")
              .email("test1@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 2)).build())
                      .collect(Collectors.toCollection(HashSet::new)))
              .build();
      userRepository.save(user);

      user = UserEntity.builder().username("test2").password(passwordEncoder.encode("qwer5795")).nickname("test2")
              .email("test1@example.com").block(false).resetCount(0L).sendEmail(true).roles(Stream.of(RoleEntity.builder().id(Short.valueOf((short) 2)).build())
                      .collect(Collectors.toCollection(HashSet::new)))
              .build();

      userRepository.save(user);
    }

    log.debug("-- Database has been initialized");
  }
}
